package com.gxzc.zen.config

import com.alibaba.druid.pool.DruidDataSource
import com.alibaba.druid.support.http.StatViewServlet
import com.alibaba.druid.support.http.WebStatFilter
import org.mybatis.spring.annotation.MapperScan
import org.slf4j.LoggerFactory
import org.springframework.boot.context.properties.ConfigurationProperties
import org.springframework.boot.web.servlet.FilterRegistrationBean
import org.springframework.boot.web.servlet.ServletRegistrationBean
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import java.sql.SQLException
import javax.sql.DataSource


@Configuration
@ConfigurationProperties(prefix = "druid.datasource-sys")
@MapperScan(basePackages = ["com.gxzc.zen.sys"])
class SysDruidConfig:DruidConfig()





/*-----------------druid数据源bean注册 */
open class DruidConfig {
    var url:String?=null
    var username:String?=null
    var password:String?=null
    var filters:String?=null

    var driverClassName:String?="com.mysql.jdbc.Driver"
    var initialSize:Int?=5
    var minIdle:Int?=5
    var maxActive:Int?=20
    var maxWait:Long?=60000

    var timeBetweenEvictionRunsMillis:Long?=null
    var minEvictableIdleTimeMillis:Long?=null
    var validationQuery:String?=null
    var testWhileIdle:Boolean?=null
    var testOnBorrow:Boolean?=null
    var testOnReturn:Boolean?=null

    private val logger= LoggerFactory.getLogger(javaClass)

    private var datasource:DruidDataSource?=null

//    private val name=nm

    @Bean
    open fun druidDataSource(): DataSource {
        datasource = DruidDataSource()
        datasource?.url = url
        datasource?.username = username
        datasource?.password = password
        datasource?.driverClassName = driverClassName
        datasource?.initialSize = initialSize!!
        datasource?.minIdle = minIdle!!
        datasource?.maxActive = maxActive!!
        datasource?.maxWait = maxWait!!
        datasource?.timeBetweenEvictionRunsMillis = timeBetweenEvictionRunsMillis!!
        datasource?.minEvictableIdleTimeMillis = minEvictableIdleTimeMillis!!
        datasource?.validationQuery = validationQuery!!
        datasource?.isTestWhileIdle = testWhileIdle!!
        datasource?.isTestOnBorrow = testOnBorrow!!
        datasource?.isTestOnReturn = testOnReturn!!
        try {
            datasource?.setFilters(filters)
        } catch (e: SQLException) {
            logger.error("druid configuration initialization filter", e)
        }

        return datasource!!
    }

    fun getDatasource():DruidDataSource{
        return datasource!!
    }
}




/*-----------------配置druid显示界面 */
@Configuration
@ConfigurationProperties(prefix = "druid.view")
class DruidViewConfig{
    var username:String?=null
    var password:String?=null
    var slowsql:String?=null

    @Bean
    fun druidServlet(): ServletRegistrationBean {
//        if(!enable!!)return throw Exception()
        val reg = ServletRegistrationBean()
        reg.setServlet(StatViewServlet())
        reg.addUrlMappings("/druid/*")
        reg.addInitParameter("loginUsername", username)
        reg.addInitParameter("loginPassword", password)
        reg.addInitParameter("logSlowSql", slowsql)
        return reg
    }

    @Bean
    fun filterRegistrationBean(): FilterRegistrationBean {
//        if(!enable!!)return throw Exception()
        val filterRegistrationBean = FilterRegistrationBean()
        filterRegistrationBean.filter = WebStatFilter()
        filterRegistrationBean.addUrlPatterns("/*")
        filterRegistrationBean.addInitParameter("exclusions", "*.js,*.gif,*.jpg,*.png,*.css,*.ico,/druid/*")
        filterRegistrationBean.addInitParameter("profileEnable", "true")
        return filterRegistrationBean
    }
}